/*
 * JPython Database Specification API 2.0
 *
 * $Id: DefaultDataHandler.java,v 1.4 2000/07/06 19:02:42 bzimmer Exp $
 *
 * Copyright (C) 2000 brian zimmer <bzimmer@ziclix.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package com.ziclix.python.sql;

import java.sql.*;
import java.math.*;
import org.python.core.*;

/**
 * The default type handler.  It supports all JDBC 1.x type
 * mappings.
 *
 * @author brian zimmer
 * @author last revised by $Author: bzimmer $
 * @version $Revision: 1.4 $
 */
public class DefaultDataHandler implements DataHandler {

	/**
	 * $CONSTRUCTOR_DESCRIPTION$
	 *
	 */
	public DefaultDataHandler() {}

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @param stmt
	 * @param index
	 * @param object
	 * @throws SQLException
	 */
	public void setJDBCObject(PreparedStatement stmt, int index, PyObject object) throws SQLException {
		stmt.setObject(index, object.__tojava__(Object.class));
	}

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @param stmt
	 * @param index
	 * @param object
	 * @param type
	 * @throws SQLException
	 */
	public void setJDBCObject(PreparedStatement stmt, int index, PyObject object, int type) throws SQLException {
		stmt.setObject(index, object.__tojava__(Object.class), type);
	}

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @param set
	 * @param col
	 * @param type
	 * @return
	 * @throws SQLException
	 */
	public PyObject getPyObject(ResultSet set, int col, int type) throws SQLException {
		PyObject obj = Py.None;
		switch(type) {

		case Types.CHAR:

		case Types.VARCHAR:

		case Types.LONGVARCHAR:
			obj = Py.newString(set.getString(col));

			break;

		case Types.NUMERIC:

		case Types.DECIMAL:
			BigDecimal bd = set.getBigDecimal(col, 10);
			obj = bd == null ? Py.None : Py.newFloat(bd.doubleValue());

			break;

		case Types.BIT:
			obj = set.getBoolean(col) ? Py.newInteger(1) : Py.newInteger(0);

			break;

		case Types.INTEGER:

		case Types.TINYINT:

		case Types.SMALLINT:
			obj = Py.newInteger(set.getInt(col));

			break;

		case Types.BIGINT:
			obj = new PyLong(set.getLong(col));

			break;

		case Types.FLOAT:

		case Types.REAL:

		case Types.DOUBLE:
			obj = Py.newFloat(set.getFloat(col));

			break;

		case Types.TIME:
			obj = Py.java2py(set.getTime(col));

			break;

		case Types.TIMESTAMP:
			obj = Py.java2py(set.getTimestamp(col));

			break;

		case Types.DATE:
			obj = Py.java2py(set.getDate(col));

			break;

		case Types.NULL:
			obj = Py.None;

			break;

		case Types.OTHER:
			obj = Py.java2py(set.getObject(col));

			break;

		case Types.BINARY:

		case Types.VARBINARY:

		case Types.LONGVARBINARY:
			obj = Py.java2py(set.getObject(col));

			break;

		default:
			throw new SQLException("unable to map type: " + type + " for column: " + col);
		}

		return (set.wasNull() || obj == null) ? Py.None : obj;
	}
}

