/*
 * JPython Database Specification API 2.0
 *
 * $Id: JDBC20DataHandler.java,v 1.3 2000/07/06 00:24:02 bzimmer Exp $
 *
 * Copyright (C) 2000 brian zimmer <bzimmer@ziclix.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package com.ziclix.python.sql;

import java.sql.*;
import java.math.*;
import org.python.core.*;

/**
 * Support for JDBC 2.x type mappings, including Arrays, CLOBs and BLOBs.
 *
 * @author brian zimmer
 * @author last revised by $Author: bzimmer $
 * @version $Revision: 1.3 $
 */
public class JDBC20DataHandler extends DefaultDataHandler {

	/**
	 * $CONSTRUCTOR_DESCRIPTION$
	 *
	 */
	public JDBC20DataHandler() {}

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @param set
	 * @param col
	 * @param type
	 * @return
	 * @throws SQLException
	 */
	public PyObject getPyObject(ResultSet set, int col, int type) throws SQLException {
		PyObject obj = Py.None;
		switch(type) {

		case Types.NUMERIC:

		case Types.DECIMAL:

			// in JDBC 2.0, use of a scale is deprecated
			BigDecimal bd = set.getBigDecimal(col);
			obj = bd == null ? Py.None : Py.newFloat(bd.doubleValue());

			break;

		case Types.CLOB:
			obj = Py.newString(set.getString(col));

			break;

		case Types.BLOB:
			obj = Py.java2py(set.getObject(col));

			break;

		case Types.ARRAY:
			obj = Py.java2py(set.getArray(col).getArray());

			break;

		default:
			return super.getPyObject(set, col, type);
		}

		return (set.wasNull() || obj == null) ? Py.None : obj;
	}
}

