/*
 * JPython Database Specification API 2.0
 *
 * $Id: PyExtendedCursor.java,v 1.3 2000/07/06 19:02:42 bzimmer Exp $
 *
 * Copyright (C) 2000 brian zimmer <bzimmer@ziclix.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package com.ziclix.python.sql;

import java.io.*;
import java.sql.*;
import java.util.*;
import org.python.core.*;

/**
 * A cursor with extensions to the DB API 2.0.
 *
 * @author brian zimmer
 * @author last revised by $Author: bzimmer $
 * @version $Revision: 1.3 $
 */
public class PyExtendedCursor extends PyCursor {

	// __class__ boilerplate -- see PyObject for details
	public static PyClass __class__;

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @return
	 */
	protected PyClass getPyClass() {
		return __class__;
	}

	protected static PyList __methods__;
	static {
		PyObject[] m = {
			new PyString("tables"), new PyString("columns"), new PyString("primarykeys"), new PyString("foreignkeys"),
			new PyString("procedures"), new PyString("procedurecolumns"), new PyString("statistics")
		};
		__methods__ = new PyList(m);
	}

	protected static PyList __members__;
	static {
		PyObject[] m = {};
		__methods__ = new PyList(m);
	}

	/**
	 * String representation of the object.
	 *
	 * @return a string representation of the object.
	 */
	public String toString() {
		return "<PyExtendedCursor object instance at " + hashCode() + ">";
	}

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @param dict
	 */
	public void initModule(PyObject dict) {
		super.initModule(dict);
		dict.__setitem__("__version__",
				Py.newString("$Revision: 1.3 $").__getslice__(Py.newInteger(11), Py.newInteger(-2), null));
		dict.__setitem__("tables", new ExtendedCursorFunc("tables", 100, 4, 4, "query for table information"));
		dict.__setitem__("columns", new ExtendedCursorFunc("columns", 101, 4, 4, "query for column information"));
		dict.__setitem__("primarykeys", new ExtendedCursorFunc("primarykeys", 102, 3, 3, "query for primary keys"));
		dict.__setitem__("foreignkeys", new ExtendedCursorFunc("foreignkeys", 103, 6, 6, "query for foreign keys"));
		dict.__setitem__("procedures", new ExtendedCursorFunc("procedures", 104, 3, 3, "query for procedures"));
		dict.__setitem__("procedurecolumns",
				new ExtendedCursorFunc("procedurecolumns", 105, 4, 4, "query for procedures columns"));
		dict.__setitem__("statistics",
				new ExtendedCursorFunc("statistics", 106, 5, 5, "description of a table's indices and statistics"));
		dict.__setitem__("initModule", null);
		dict.__setitem__("__class__", null);
	}

	/**
	 * Only table descriptions matching the catalog, schema, table name and type
	 * criteria are returned. They are ordered by TABLE_TYPE, TABLE_SCHEM and
	 * TABLE_NAME.
	 *
	 * @param qualifier
	 * @param owner
	 * @param table
	 * @param type
	 */
	public void tables(PyObject qualifier, PyObject owner, PyObject table, PyObject type) {
		clear();

		try {
			String q = qualifier == Py.None ? null : (String) qualifier.__tojava__(String.class);
			String o = owner == Py.None ? null : (String) owner.__tojava__(String.class);
			String t = table == Py.None ? null : (String) table.__tojava__(String.class);
			String[] y = null;
			if(type != Py.None) {
				if(PyUtil.PyTupleList_Check(type)) {
					int len = type.__len__();
					y = new String[len];

					for(int i = 0; i < len; i++) {
						y[i] = (String) type.__getitem__(i).__tojava__(String.class);
					}
				} else {
					y = new String[1];
					y[0] = (String) type.__tojava__(String.class);
				}
			}

			create(this.connection.getMetaData().getTables(q, o, t, y));
		} catch(SQLException e) {
			zxJDBC.Error(e);
		}
	}

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @param qualifier
	 * @param owner
	 * @param table
	 * @param column
	 */
	public void columns(PyObject qualifier, PyObject owner, PyObject table, PyObject column) {
		clear();

		try {
			String q = qualifier == Py.None ? null : (String) qualifier.__tojava__(String.class);
			String o = owner == Py.None ? null : (String) owner.__tojava__(String.class);
			String t = table == Py.None ? null : (String) table.__tojava__(String.class);
			String c = column == Py.None ? null : (String) column.__tojava__(String.class);
			create(this.connection.getMetaData().getColumns(q, o, t, c));
		} catch(SQLException e) {
			zxJDBC.Error(e);
		}
	}

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @param qualifier
	 * @param owner
	 * @param procedure
	 */
	public void procedures(PyObject qualifier, PyObject owner, PyObject procedure) {
		clear();

		try {
			String q = qualifier == Py.None ? null : (String) qualifier.__tojava__(String.class);
			String o = owner == Py.None ? null : (String) owner.__tojava__(String.class);
			String p = procedure == Py.None ? null : (String) procedure.__tojava__(String.class);
			create(this.connection.getMetaData().getProcedures(q, o, p));
		} catch(SQLException e) {
			zxJDBC.Error(e);
		}
	}

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @param qualifier
	 * @param owner
	 * @param procedure
	 * @param column
	 */
	public void procedurecolumns(PyObject qualifier, PyObject owner, PyObject procedure, PyObject column) {
		clear();

		try {
			String q = qualifier == Py.None ? null : (String) qualifier.__tojava__(String.class);
			String o = owner == Py.None ? null : (String) owner.__tojava__(String.class);
			String p = procedure == Py.None ? null : (String) procedure.__tojava__(String.class);
			String c = column == Py.None ? null : (String) column.__tojava__(String.class);
			create(this.connection.getMetaData().getProcedureColumns(q, o, p, c));
		} catch(SQLException e) {
			zxJDBC.Error(e);
		}
	}

	/**
	 * Gets a description of a table's primary key columns. They are ordered by
	 * COLUMN_NAME.
	 *
	 * @param qualifier a schema name
	 * @param owner an owner name
	 * @param table a table name
	 */
	public void primarykeys(PyObject qualifier, PyObject owner, PyObject table) {
		clear();

		try {
			String q = qualifier == Py.None ? null : (String) qualifier.__tojava__(String.class);
			String o = owner == Py.None ? null : (String) owner.__tojava__(String.class);
			String t = table == Py.None ? null : (String) table.__tojava__(String.class);
			create(this.connection.getMetaData().getPrimaryKeys(q, o, t));
		} catch(SQLException e) {
			zxJDBC.Error(e);
		}
	}

	/**
	 * Gets a description of the foreign key columns in the foreign key table
	 * that reference the primary key columns of the primary key table (describe
	 * how one table imports another's key.) This should normally return a single
	 * foreign key/primary key pair (most tables only import a foreign key from a
	 * table once.) They are ordered by FKTABLE_CAT, FKTABLE_SCHEM, FKTABLE_NAME,
	 * and KEY_SEQ.
	 *
	 * @param primaryQualifier
	 * @param primaryOwner
	 * @param primaryTable
	 * @param foreignQualifier
	 * @param foreignOwner
	 * @param foreignTable
	 */
	public void foreignkeys(PyObject primaryQualifier, PyObject primaryOwner, PyObject primaryTable,
			PyObject foreignQualifier, PyObject foreignOwner, PyObject foreignTable) {
		clear();

		try {
			String pq = primaryQualifier == Py.None ? null : (String) primaryQualifier.__tojava__(String.class);
			String po = primaryOwner == Py.None ? null : (String) primaryOwner.__tojava__(String.class);
			String pt = primaryTable == Py.None ? null : (String) primaryTable.__tojava__(String.class);
			String fq = foreignQualifier == Py.None ? null : (String) foreignQualifier.__tojava__(String.class);
			String fo = foreignOwner == Py.None ? null : (String) foreignOwner.__tojava__(String.class);
			String ft = foreignTable == Py.None ? null : (String) foreignTable.__tojava__(String.class);
			create(this.connection.getMetaData().getCrossReference(pq, po, pt, fq, fo, ft));
		} catch(SQLException e) {
			zxJDBC.Error(e);
		}
	}

	/**
	 * Gets a description of a table's indices and statistics. They are ordered by
	 * NON_UNIQUE, TYPE, INDEX_NAME, and ORDINAL_POSITION.
	 *
	 * @param qualifier
	 * @param owner
	 * @param table
	 * @param unique
	 * @param accuracy
	 */
	public void statistics(PyObject qualifier, PyObject owner, PyObject table, PyObject unique, PyObject accuracy) {
		clear();

		try {
			String q = qualifier == Py.None ? null : (String) qualifier.__tojava__(String.class);
			String o = owner == Py.None ? null : (String) owner.__tojava__(String.class);
			String t = table == Py.None ? null : (String) table.__tojava__(String.class);
			boolean u = unique == Py.None ? false : ((Boolean) unique.__tojava__(Boolean.class)).booleanValue();
			boolean a = accuracy == Py.None ? false : ((Boolean) accuracy.__tojava__(Boolean.class)).booleanValue();
			create(this.connection.getMetaData().getIndexInfo(q, o, t, u, a));
		} catch(SQLException e) {
			zxJDBC.Error(e);
		}
	}

	/**
	 * $CLASS_DESCRIPTION$
	 *
	 * @author
	 * @author last revised by $Author: bzimmer $
	 * @version $Revision: 1.3 $
	 */
	class ExtendedCursorFunc extends PyBuiltinFunctionSet {

		/**
		 * $CONSTRUCTOR_DESCRIPTION$
		 *
		 * @param name
		 * @param index
		 * @param argcount
		 * @param doc
		 */
		ExtendedCursorFunc(String name, int index, int argcount, String doc) {
			super(name, index, argcount, argcount, true, doc);
		}

		/**
		 * $CONSTRUCTOR_DESCRIPTION$
		 *
		 * @param name
		 * @param index
		 * @param minargs
		 * @param maxargs
		 * @param doc
		 */
		ExtendedCursorFunc(String name, int index, int minargs, int maxargs, String doc) {
			super(name, index, minargs, maxargs, true, doc);
		}

		/**
		 * $METHOD_DESCRIPTION$
		 *
		 * @param arga
		 * @param argb
		 * @param argc
		 * @return
		 */
		public PyObject __call__(PyObject arga, PyObject argb, PyObject argc) {
			PyExtendedCursor cursor = (PyExtendedCursor) __self__;
			switch(index) {

			case 102:
				cursor.primarykeys(arga, argb, argc);

				return Py.None;

			case 104:
				cursor.procedures(arga, argb, argc);

				return Py.None;

			default:
				throw argCountError(3);
			}
		}

		/**
		 * $METHOD_DESCRIPTION$
		 *
		 * @param args
		 * @return
		 */
		public PyObject fancyCall(PyObject[] args) {
			PyExtendedCursor cursor = (PyExtendedCursor) __self__;
			switch(index) {

			case 103:
				cursor.foreignkeys(args[0], args[1], args[2], args[3], args[4], args[5]);

				return Py.None;

			case 106:
				cursor.statistics(args[0], args[1], args[2], args[3], args[4]);

				return Py.None;

			default:
				throw argCountError(args.length);
			}
		}

		/**
		 * $METHOD_DESCRIPTION$
		 *
		 * @param arg1
		 * @param arg2
		 * @param arg3
		 * @param arg4
		 * @return
		 */
		public PyObject __call__(PyObject arg1, PyObject arg2, PyObject arg3, PyObject arg4) {
			PyExtendedCursor cursor = (PyExtendedCursor) __self__;
			switch(index) {

			case 100:
				cursor.tables(arg1, arg2, arg3, arg4);

				return Py.None;

			case 101:
				cursor.columns(arg1, arg2, arg3, arg4);

				return Py.None;

			case 105:
				cursor.procedurecolumns(arg1, arg2, arg3, arg4);

				return Py.None;

			default:
				throw argCountError(4);
			}
		}
	}

}

