/*
 * JPython Database Specification API 2.0
 *
 * $Id: PyUtil.java,v 1.4 2000/07/06 19:02:42 bzimmer Exp $
 *
 * Copyright (C) 2000 brian zimmer <bzimmer@ziclix.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package com.ziclix.python.sql;

import java.io.*;
import java.sql.*;
import java.math.*;
import java.util.*;
import org.python.core.*;

/**
 * Static utility class.
 *
 * @author brian zimmer
 * @author last revised by $Author: bzimmer $
 * @version $Revision: 1.4 $
 */
public class PyUtil {

	/**
	 * $CONSTRUCTOR_DESCRIPTION$
	 *
	 */
	private PyUtil() {}

	/**
	 * Print the debug message.
	 *
	 * @param message
	 */
	public static void debug(String message) {
		Py.println(new PyString(message));
	}

	/**
	 * Is the obj a List?
	 *
	 * @param obj
	 * @return
	 */
	public static boolean PyList_Check(PyObject obj) {
		return obj instanceof PyList;
	}

	/**
	 * Is the obj a Tuple?
	 *
	 * @param obj
	 * @return
	 */
	public static boolean PyTuple_Check(PyObject obj) {
		return obj instanceof PyTuple;
	}

	/**
	 * Is the obj a List or Tuple?
	 *
	 * @param obj
	 * @return
	 */
	public static boolean PyTupleList_Check(PyObject obj) {
		return PyTuple_Check(obj) || PyList_Check(obj);
	}

	/**
	 * Is the obj a List or Tuple of Lists or Tuples?
	 *
	 * @param obj
	 * @return
	 */
	public static boolean PyTupleListOfTupleList_Check(PyObject obj) {
		boolean ok = true;
		if(obj == null) {
			return false;
		}

		for(int i = 0; ok && i < obj.__len__(); i++) {
			ok = PyTupleList_Check(obj.__getitem__(i));
		}

		return ok;
	}
}

