/*
 * JPython Database Specification API 2.0
 *
 * $Id: InformixDataHandler.java,v 1.5 2000/07/06 19:02:43 bzimmer Exp $
 *
 * Copyright (C) 2000 brian zimmer <bzimmer@ziclix.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package com.ziclix.python.sql.informix;

import java.sql.*;
import java.math.*;
import org.python.core.*;
import com.ziclix.python.sql.*;

/**
 * Informix specific data handling.
 *
 * @author brian zimmer
 * @author last revised by $Author: bzimmer $
 * @version $Revision: 1.5 $
 */
public class InformixDataHandler implements DataHandler {
	protected DataHandler datahandler;

	/**
	 * $CONSTRUCTOR_DESCRIPTION$
	 *
	 * @param datahandler
	 */
	public InformixDataHandler(DataHandler datahandler) {
		this.datahandler = datahandler;
	}

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @param stmt
	 * @param index
	 * @param object
	 * @throws SQLException
	 */
	public void setJDBCObject(PreparedStatement stmt, int index, PyObject object) throws SQLException {
		this.datahandler.setJDBCObject(stmt, index, object);
	}

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @param stmt
	 * @param index
	 * @param object
	 * @param type
	 * @throws SQLException
	 */
	public void setJDBCObject(PreparedStatement stmt, int index, PyObject object, int type) throws SQLException {
		if(type == Types.OTHER) {
			Object integer = object.__tojava__(Integer.class);
			if(!integer.equals(Py.NoConversion)) {

				// probably a boolean
				stmt.setBoolean(index, ((Integer) integer).intValue() != 0 ? true : false);
			}
		}

		this.datahandler.setJDBCObject(stmt, index, object, type);
	}

	/**
	 * $METHOD_DESCRIPTION$
	 *
	 * @param set
	 * @param col
	 * @param type
	 * @return
	 * @throws SQLException
	 */
	public PyObject getPyObject(ResultSet set, int col, int type) throws SQLException {
		PyObject obj = Py.None;
		if(type == Types.OTHER) {
			try {

				// informix returns boolean as OTHERs, so let's give that a try
				obj = set.getBoolean(col) ? new PyInteger(1) : new PyInteger(0);

				return (set.wasNull() || obj == null) ? Py.None : obj;
			} catch(SQLException e) {}
		}

		return this.datahandler.getPyObject(set, col, type);
	}
}

