/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * JPython Database Specification API 2.0
 * 
 * $Id: zxJDBC.java,v 1.13 2000/07/07 15:01:08 bzimmer Exp $
 * 
 * Copyright (C) 2000 brian zimmer <bzimmer@ziclix.com>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
package com.ziclix.python.sql;

import java.io.*;
import java.sql.*;
import java.lang.reflect.*;
import java.util.Properties;
import java.util.Calendar;
import org.python.core.*;

/**
 * Creates database connections.
 * 
 * <pre>
 * from com.ziclix.python.sql import zxJDBC
 * db = zxJDBC.connect("jdbc:mysql://localhost:3306/MySql", None, None, "org.gjt.mm.mysql.Driver")
 * </pre>
 * 
 * @author brian zimmer
 * @author last revised by $Author: bzimmer $
 * @version $Revision: 1.13 $
 */
public class zxJDBC extends PyObject implements InitModule {

	/**
	 * Initializes the module.
	 * 
	 * @param dict
	 */
	public void initModule(PyObject dict) {
		dict.__setitem__("apilevel", new PyString("2.0"));
		dict.__setitem__("threadsafety", new PyInteger(1));
		dict.__setitem__("paramstyle", new PyString("qmark"));
		dict.__setitem__("__version__", 
				Py.newString("$Revision: 1.13 $").__getslice__(Py.newInteger(11), Py.newInteger(-2), null));
		dict.__setitem__("Error", zxJDBC.Error);
		dict.__setitem__("DatabaseError", zxJDBC.DatabaseError);
		dict.__setitem__("NotSupportedError", zxJDBC.NotSupportedError);
		dict.__setitem__("InterfaceError", zxJDBC.InterfaceError);
		dict.__setitem__("Warning", zxJDBC.Warning);
		dict.__setitem__("OperationalError", zxJDBC.OperationalError);
		dict.__setitem__("ProgrammingError", zxJDBC.ProgrammingError);
		dict.__setitem__("DataError", zxJDBC.DataError);
		dict.__setitem__("connect", new zxJDBCFunc("connect", 0, 1, 4, true, "connect to the db"));
		dict.__setitem__("Date", new zxJDBCFunc("Date", 1, 3, 3, false, "construct a Date from year, month, day"));
		dict.__setitem__("Time", new zxJDBCFunc("Time", 2, 3, 3, false, "construct a Date from hour, minute, second"));
		dict.__setitem__("Timestamp", 
				new zxJDBCFunc("Timestamp", 3, 6, 6, false, 
				"construct a Timestamp from year, month, day, hour, minute, second"));
		dict.__setitem__("DateFromTicks", 
				new zxJDBCFunc("DateFromTicks", 4, 1, 1, false, "construct a Date from seconds since the epoch"));
		dict.__setitem__("TimeFromTicks", 
				new zxJDBCFunc("TimeFromTicks", 5, 1, 1, false, "construct a Time from seconds since the epoch"));
		dict.__setitem__("TimestampFromTicks", 
				new zxJDBCFunc("TimestampFromTicks", 6, 1, 1, false, 
				"construct a Timestamp from seconds since the epoch"));
		addSqlTypes(dict);

		// hide from python
		dict.__setitem__("initModule", null);
		dict.__setitem__("__class__", null);
		dict.__setitem__("toString", null);
	} 

	// __class__ boilerplate -- see PyObject for details
	public static PyClass __class__;
	protected PyClass getPyClass() {
		return __class__;
	} 
	protected static void addSqlTypes(PyObject dict) throws PyException {
		PyDictionary sqltype = new PyDictionary();
		dict.__setitem__("sqltype", sqltype);

		try {
			Class c = Class.forName("java.sql.Types");
			Field[] fields = c.getFields();
			for(int i = 0; i < fields.length; i++) {
				Field f = fields[i];
				dict.__setitem__(Py.newString(f.getName()), new PyInteger(f.getInt(c)));
				sqltype.__setitem__(new PyInteger(f.getInt(c)), Py.newString(f.getName()));
			} 
		} catch(Exception e) {
			throw Error(e.getMessage());
		} 
	} 

	public static PyObject Error = new PyString("Error");
	public static PyException Error(String msg) {
		return new PyException(Error, msg);
	} 
	protected static PyException Error(SQLException e) {
		return Error(e.getMessage() + " [SQLCode: " + e.getErrorCode() + "]");
	} 

	public static PyObject DatabaseError = new PyString("DatabaseError");
	public static PyException DatabaseError(String msg) {
		return new PyException(DatabaseError, msg);
	} 

	public static PyObject NotSupportedError = new PyString("NotSupportedError");
	public static PyException NotSupportedError(String msg) {
		return new PyException(NotSupportedError, msg);
	} 

	public static PyObject ProgrammingError = new PyString("ProgrammingError");
	public static PyException ProgrammingError(String msg) {
		return new PyException(ProgrammingError, msg);
	} 

	public static PyObject OperationalError = new PyString("OperationalError");
	public static PyException OperationalError(String msg) {
		return new PyException(OperationalError, msg);
	} 

	public static PyObject DataError = new PyString("DataError");
	public static PyException DataError(String msg) {
		return new PyException(DataError, msg);
	} 

	public static PyObject InterfaceError = new PyString("InterfaceError");
	public static PyException InterfaceError(String msg) {
		return new PyException(InterfaceError, msg);
	} 

	public static PyObject Warning = new PyString("Warning");
	public static PyException Warning(String msg) {
		return new PyException(Warning, msg);
	} 

	/**
	 * Connects to the database.
	 * 
	 * @param url connection url
	 * @param props Properties to be passed to driver
	 * @param driver valid JDBC driver
	 * @return connection to the database
	 */
	public static PyConnection connect(String url, Properties props, String driver) {
		Connection c = null;
		try {
			Class.forName(driver).newInstance();
		} catch(ClassNotFoundException e) {
			throw DatabaseError("Driver not found.");
		} catch(InstantiationException e) {
			throw DatabaseError("Unable to instantiate driver.");
		} catch(IllegalAccessException e) {
			throw DatabaseError("Unable to instantiate driver.");
		} 

		try {
			c = DriverManager.getConnection(url, props);

			if(c != null &&!c.isClosed()) {
				PyConnection pc = new PyConnection();
				pc.setConnection(c);

				return pc;
			} 
		} catch(SQLException e) {
			throw DatabaseError(e.getMessage());
		} 

		throw DatabaseError("Unable to connect.");
	} 

	/**
	 * This function constructs an object holding a date value.
	 */
	public static PyObject Date(int year, int month, int day) {
		Calendar c = Calendar.getInstance();
		c.set(Calendar.YEAR, year);
		c.set(Calendar.MONTH, month - 1);
		c.set(Calendar.DATE, day);

		return DateFromTicks(c.getTime().getTime() / 1000.0);
	} 

	/**
	 * This function constructs an object holding a time value.
	 */
	public static PyObject Time(int hour, int minute, int second) {
		Calendar c = Calendar.getInstance();
		c.set(Calendar.HOUR, hour);
		c.set(Calendar.MINUTE, minute);
		c.set(Calendar.SECOND, second);

		return TimeFromTicks(c.getTime().getTime() / 1000.0);
	} 

	/**
	 * This function constructs an object holding a time stamp value.
	 */
	public static PyObject Timestamp(int year, int month, int day, int hour, int minute, int second) {
		Calendar c = Calendar.getInstance();
		c.set(Calendar.YEAR, year);
		c.set(Calendar.MONTH, month - 1);
		c.set(Calendar.DATE, day);
		c.set(Calendar.HOUR, hour);
		c.set(Calendar.MINUTE, minute);
		c.set(Calendar.SECOND, second);
		c.set(Calendar.MILLISECOND, 0);

		return TimestampFromTicks(c.getTime().getTime() / 1000.0);
	} 

	/**
	 * This function constructs an object holding a date value from the
	 * given ticks value (number of seconds since the epoch; see the
	 * documentation of the standard Python <i>time</i> module for details).
	 * 
	 * <i>Note:</i> The DB API 2.0 spec calls for time in seconds since the epoch
	 * while the Java Date object returns time in milliseconds since the epoch.
	 * This module adheres to the python API and will therefore use time in
	 * seconds rather than milliseconds, so adjust any Java code accordingly.
	 * 
	 * @param ticks number of seconds since the epoch
	 */
	public static PyObject DateFromTicks(double ticks) {
		return new PyJavaInstance(new Date((long) (ticks * 1000)));
	} 

	/**
	 * This function constructs an object holding a time value from the
	 * given ticks value (number of seconds since the epoch; see the
	 * documentation of the standard Python <i>time</i> module for details).
	 * 
	 * <i>Note:</i> The DB API 2.0 spec calls for time in seconds since the epoch
	 * while the Java Date object returns time in milliseconds since the epoch.
	 * This module adheres to the python API and will therefore use time in
	 * seconds rather than milliseconds, so adjust any Java code accordingly.
	 * 
	 * @param ticks number of seconds since the epoch
	 */
	public static PyObject TimeFromTicks(double ticks) {
		return new PyJavaInstance(new Time((long) (ticks * 1000)));
	} 

	/**
	 * This function constructs an object holding a time stamp value from
	 * the given ticks value (number of seconds since the epoch; see the
	 * documentation of the standard Python <i>time</i> module for details).
	 * 
	 * <i>Note:</i> The DB API 2.0 spec calls for time in seconds since the epoch
	 * while the Java Date object returns time in milliseconds since the epoch.
	 * This module adheres to the python API and will therefore use time in
	 * seconds rather than milliseconds, so adjust any Java code accordingly.
	 * 
	 * @param ticks number of seconds since the epoch
	 */
	public static PyObject TimestampFromTicks(double ticks) {
		return new PyJavaInstance(new Timestamp((long) (ticks * 1000)));
	} 

	class zxJDBCFunc extends PyBuiltinFunctionSet {
		zxJDBCFunc(String name, int index, int minargs, int maxargs, boolean func, String doc) {
			super(name, index, minargs, maxargs, func, doc);
		}

		public PyObject __call__(PyObject arg) {
			double ticks = ((Number) arg.__tojava__(Number.class)).doubleValue();
			switch(index) {

			case 4:
				return DateFromTicks(ticks);

			case 5:
				return TimeFromTicks(ticks);

			case 6:
				return TimestampFromTicks(ticks);

			default:
				throw argCountError(1);
			}
		} 
		public PyObject __call__(PyObject[] args) {
			if(index == 0) {
				return __call__(args, Py.NoKeywords);
			} 

			return super.__call__(args);
		} 
		public PyObject __call__(PyObject arga, PyObject argb, PyObject argc) {
			switch(index) {

			case 1:
				int year = ((Number) arga.__tojava__(Number.class)).intValue();
				int month = ((Number) argb.__tojava__(Number.class)).intValue();
				int day = ((Number) argc.__tojava__(Number.class)).intValue();
				return Date(year, month, day);

			case 2:
				int hour = ((Number) arga.__tojava__(Number.class)).intValue();
				int minute = ((Number) argb.__tojava__(Number.class)).intValue();
				int second = ((Number) argc.__tojava__(Number.class)).intValue();
				return Time(hour, minute, second);

			default:
				throw argCountError(3);
			}
		} 
		public PyObject fancyCall(PyObject[] args) {
			switch(index) {

			case 3:
				int year = ((Number) args[0].__tojava__(Number.class)).intValue();
				int month = ((Number) args[1].__tojava__(Number.class)).intValue();
				int day = ((Number) args[2].__tojava__(Number.class)).intValue();
				int hour = ((Number) args[3].__tojava__(Number.class)).intValue();
				int minute = ((Number) args[4].__tojava__(Number.class)).intValue();
				int second = ((Number) args[5].__tojava__(Number.class)).intValue();
				return Timestamp(year, month, day, hour, minute, second);

			default:
				throw argCountError(args.length);
			}
		} 
		public PyObject __call__(PyObject[] args, String[] keywords) {

			/*
			 * The first len(args)-len(keywords) members of args[] are plain
			 * arguments.  The last len(keywords) arguments are the values of
			 * the keyword arguments.
			 */
			int largs = args.length;
			int lkeywords = keywords.length;
			int diff = largs - lkeywords;
			String url = (String) args[0].__tojava__(String.class);
			String user = (String) args[1].__tojava__(String.class);
			String password = (String) args[2].__tojava__(String.class);
			String driver = (String) args[3].__tojava__(String.class);
			Properties props = new Properties();

			// the value can't be null
			props.put("user", user == null ? "" : user);
			props.put("password", password == null ? "" : password);

			for(int i = 0; i < lkeywords; i++) {
				String kw = keywords[i];
				Object value = args[diff + i].__tojava__(Object.class);
				props.put(kw, value);
			} 

			return connect(url, props, driver);
		} 
	}

}

